--[[
Author: Mantrid
https://fs.mantrid.net/
Copyright: All rights reserved
LEGAL NOTICE: This mod, and this file in particular, are only authorised for distribution by GIANTS Software. All other entities agree to pay the sum of AU$1,000,000 for distribution rights.
Aspiring developers: I have limited experience with coding LUA, so my methods may not be the best. You're welcome to read and play with the code, for the purposes of learning, but do not copy whole functions for your own projects.
Experienced developers: It's my first day :)
]]

newMoon = {}

newMoon.buttonId = {}

newMoon.saveDataLoaded = false
newMoon.isEnabled = false

function newMoon:loadMap(savegame)
	Player.registerActionEvents = Utils.appendedFunction(Player.registerActionEvents, newMoon.updateControlsMenu)
	Player.removeActionEvents = Utils.appendedFunction(Player.removeActionEvents, newMoon.removeActionEvents)
	Lighting.updateCurves = Utils.appendedFunction(Lighting.updateCurves, newMoon.updateControlsMenu)
	FSCareerMissionInfo.saveToXMLFile = Utils.appendedFunction(FSCareerMissionInfo.saveToXMLFile, newMoon.saveSettings)
	Mission00.onStartMission = Utils.appendedFunction(Mission00.onStartMission, newMoon.loadSettings)

	Logging.info("New Moon mod has been loaded")
end

function newMoon:callbackMenu_main(what, axisComponent)
	local skyBrightness, _, _ = g_currentMission.environment.lighting.primaryExtraterrestrialColor:get(0)
	if what == "NEW_MOON_DISABLE" then
		newMoon:disable()
		newMoon:updateControlsMenu()
	elseif what == "NEW_MOON_SKYLIGHT" and axisComponent == 1 then
		local adjustment = 0.001
		if skyBrightness > 0.04 then
			adjustment = 0.002
		end
		newMoon:shootTheMoon(math.min(skyBrightness + adjustment, 0.09))
		newMoon:updateControlsMenu()
	elseif what == "NEW_MOON_SKYLIGHT" and axisComponent == -1 then
		local adjustment = 0.001
		if skyBrightness > 0.04 then
			adjustment = 0.002
		end
		newMoon:shootTheMoon(math.max(skyBrightness - adjustment, 0))
		newMoon:updateControlsMenu()
	else
		Logging.info("Unknown menuCallback: " .. tostring(what))
	end
end

function newMoon:disable()
	local env = g_currentMission.environment

	local xmlFile = XMLFile.load("Environment", env.xmlFilename)
	local lighting = Lighting.new(env)
	lighting:load(xmlFile, "environment.lighting")
	xmlFile:delete()

	env:setCustomLighting(lighting)
	newMoon.isEnabled = false
end

function newMoon:shootTheMoon(howMuch)
	if not newMoon.isEnabled then
		newMoon.isEnabled = true
	end
	local lighting = g_currentMission.environment.lighting

	local skyBrightness = howMuch -- Default night is 0.04
	local moonBrightness = 0 -- Default is 0.5 at sunset, to 4 at midnight, and back to 0.5 at sunrise

	local newMoonBrightnessScaleCurveData = {{0, {moonBrightness}}, {24, {moonBrightness}}}
	lighting.moonBrightnessScaleCurveData = newMoonBrightnessScaleCurveData
	lighting.moonBrightnessScaleCurve = lighting:createCurve(linearInterpolator1, lighting.moonBrightnessScaleCurveData)

	local newMoonPECCD = {}
	for _, data in ipairs(lighting.primaryExtraterrestrialColorCurveData) do
		local t, vals = data[1], data[2]
		if vals[1] <= 0.1 then
			vals = {skyBrightness, skyBrightness, skyBrightness}
		end
		table.insert(newMoonPECCD, { t, vals })
	end
	lighting.primaryExtraterrestrialColorCurveData = newMoonPECCD
	lighting.primaryExtraterrestrialColor = lighting:createCurve(linearInterpolator3, lighting.primaryExtraterrestrialColorCurveData)


	local newMoonAFCD = {}
	local lastTime = 0
	-- Magic for loop which requires as much commenting as code.. and yet just has this :)
	for _, data in ipairs(lighting.asymmetryFactorCurveData) do
		local t, val = data[1], data[2][1]
		if t == 0 then
			table.insert(newMoonAFCD, { 0, {0} })
		elseif t == 24 then
			if lastTime ~= nil and lastTime ~= 0 and val ~= 0 then
				table.insert(newMoonAFCD, { lastTime, {0} })
			end
			table.insert(newMoonAFCD, { 24, {0} })
		else
			if lastTime == 0 and val ~= 0 then
				table.insert(newMoonAFCD, { t, {0} })
			end
			table.insert(newMoonAFCD, { t, {val} })
		end
		lastTime = t
	end
	lighting.asymmetryFactorCurveData = newMoonAFCD
	lighting.asymmetryFactorCurve = lighting:createCurve(linearInterpolator1, lighting.asymmetryFactorCurveData)

	if newMoon.saveDataLoaded then
		lighting:updateAtmosphere(g_currentMission.environment.dayTime / 60000)
	end
end

function newMoon:updateControlsMenu()
	-- Remove all of our actions
	for key, eventId in pairs(newMoon.buttonId) do
		g_inputBinding:removeActionEvent(eventId)
		newMoon.buttonId[key] = nil
	end

	local skyBrightness = 0
	if g_currentMission.environment.lighting.primaryExtraterrestrialColor ~= nil then
		skyBrightness, _, _ = g_currentMission.environment.lighting.primaryExtraterrestrialColor:get(0)
	end
	local actions = {
		{ "NEW_MOON_SKYLIGHT", "callbackMenu_main", g_i18n:getText("newMoon_menu_skyLight") .. " " .. string.format("%.1f", skyBrightness * 100), 9 }
	}
	if newMoon.isEnabled then
		table.insert(actions, { "NEW_MOON_DISABLE", "callbackMenu_main", g_i18n:getText("newMoon_menu_disable"), 9 })
	end

	for _, item in ipairs(actions) do
		local _, eventId = nil
		if newMoon.buttonId[item[3]] == nil then
			_, eventId = g_inputBinding:registerActionEvent(InputAction[item[1]], newMoon, newMoon[item[2]], false, true, false, true)
			newMoon.buttonId[item[3]] = eventId
			g_inputBinding:setActionEventActive(eventId, true)
			g_inputBinding:setActionEventText(eventId, item[3])
			g_inputBinding:setActionEventTextPriority(eventId, item[4])
			g_inputBinding:setActionEventTextVisibility(eventId, true)
		end
	end

	if InputHelpDisplay.ENTRY_COUNT_PC < 10 then
		InputHelpDisplay.ENTRY_COUNT_PC = 10
	end
end

function newMoon:removeActionEvents()
	local g_inputBinding = g_inputBinding
	-- Remove all of our actions
	for key, eventId in pairs(newMoon.buttonId) do
		g_inputBinding:removeActionEvent(eventId)
		newMoon.buttonId[key] = nil
	end
end

function newMoon:saveSettings()
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), g_currentMission.missionInfo.savegameIndex)
	end
	local key = "newMoon"
	local xmlFile = createXMLFile(key, savegameFolderPath .. "/mod_newMoon.xml", key)

	local skyBrightness, _, _ = g_currentMission.environment.lighting.primaryExtraterrestrialColor:get(0)
	setXMLFloat(xmlFile, key .. ".skyBrightness#value", skyBrightness)
	setXMLBool(xmlFile, key .. ".modEnabled#value", newMoon.isEnabled)

	saveXMLFile(xmlFile)

	delete(xmlFile)
end

function newMoon:loadSettings()
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), g_currentMission.missionInfo.savegameIndex)
	end
	local key = "newMoon"

	if fileExists(savegameFolderPath .. "/mod_newMoon.xml") then
		if consoleLogging then Logging.info("NewMoon: Loading saved settings") end
		local xmlFile = loadXMLFile(key, savegameFolderPath .. "/mod_newMoon.xml")

		local skyBrightness = Utils.getNoNil(getXMLFloat(xmlFile, key .. ".skyBrightness#value"), 0)
		newMoon.isEnabled = Utils.getNoNil(getXMLBool(xmlFile, key .. ".modEnabled#value"), false)
		delete(xmlFile)
		if newMoon.isEnabled then
			newMoon:shootTheMoon(skyBrightness)
		end
	end
	newMoon.saveDataLoaded = true
end

addModEventListener(newMoon)